@@ -1,9 +1,12 @@
-import logging
-
+# -*- coding: utf-8 -*-
+from BTrees.Length import Length
+from plone.app.upgrade.utils import installOrReinstallProduct
 from plone.app.upgrade.utils import loadMigrationProfile
 from Products.CMFCore.utils import getToolByName
 from Products.ZCTextIndex.OkapiIndex import OkapiIndex
-from BTrees.Length import Length
+
+import logging
+
 
 logger = logging.getLogger('plone.app.upgrade')
 
@@ -14,7 +17,7 @@ def fixOkapiIndexes(catalog):
     for index in catalog.getIndexObjects():
         index = getattr(index, 'index', index)
         if isinstance(index, OkapiIndex):
-            index._totaldoclen = Length(long(sum(index._docweight.values())))
+            index._totaldoclen = Length(int(sum(index._docweight.values())))
 
 
 def fixOwnerTuples(portal):
@@ -24,11 +27,20 @@ def fixOwnerTuples(portal):
         old = obj.getOwnerTuple()
         if old and old[0][-1] == 'portal_memberdata':
             new = (['acl_users'], old[1])
-            logger.info('Repairing %s: %r -> %r' % (path, old, new))
+            logger.info('Repairing %s: %r -> %r', path, old, new)
             obj._owner = new
     portal.ZopeFindAndApply(portal, search_sub=True, apply_func=fixOwnerTuple)
 
 
+def installPloneAppDiscussion(portal):
+    # Make sure plone.app.discussion is properly installed.
+    installOrReinstallProduct(
+        portal,
+        'plone.app.discussion',
+        out=None,
+        hidden=True)
+
+
 def to411(context):
     loadMigrationProfile(context, 'profile-plone.app.upgrade.v41:4.1-4.1.1')
 
@@ -44,8 +56,23 @@ def to412_owner_tuples(context):
     portal = getToolByName(context, 'portal_url').getPortalObject()
     fixOwnerTuples(portal)
 
+
 def to413(context):
     loadMigrationProfile(context, 'profile-plone.app.upgrade.v41:4.1.2-4.1.3')
 
+
 def to414(context):
     loadMigrationProfile(context, 'profile-plone.app.upgrade.v41:4.1.3-4.1.4')
+
+
+def to415_discussion(context):
+    portal = getToolByName(context, 'portal_url').getPortalObject()
+    installPloneAppDiscussion(portal)
+
+
+def to415(context):
+    loadMigrationProfile(context, 'profile-plone.app.upgrade.v41:4.1.4-4.1.5')
+
+
+def to416(context):
+    loadMigrationProfile(context, 'profile-plone.app.upgrade.v41:4.1.5-4.1.6')
